#include <stdlib.h>
#include <stdio.h>

#include <mpi.h>

static void mpi_err_handler(int error_code, MPI_Comm comm);

static const char datarep[] = "external32";

enum {
  UINT_COUNT = 2,
  DBLE_COUNT = 27,
};

int main(int argc, char **argv) {

  MPI_Init(&argc, &argv);

  unsigned * uint_data = calloc(UINT_COUNT, sizeof(*uint_data));
  double * dble_data = calloc(DBLE_COUNT, sizeof(*dble_data));
  if (!uint_data || !dble_data) {
    fputs("failed to allocate buffers\n", stderr);
    MPI_Abort(MPI_COMM_WORLD, 1);
  }

  MPI_Aint uint_pack_size, dble_pack_size;

  mpi_err_handler(MPI_Pack_external_size(datarep, UINT_COUNT, MPI_UNSIGNED,
                                         &uint_pack_size), MPI_COMM_WORLD);
  mpi_err_handler(MPI_Pack_external_size(datarep, DBLE_COUNT, MPI_DOUBLE,
                                         &dble_pack_size), MPI_COMM_WORLD);

  MPI_Aint pack_size = uint_pack_size + dble_pack_size;

  void *pack_buffer = malloc((size_t)pack_size);
  if (!pack_buffer) {
    fprintf(stderr, "failed to allocate pack buffer to %zu chars\n",
            (size_t)pack_size);
    MPI_Abort(MPI_COMM_WORLD, 1);
  }
  MPI_Aint position = 0;

#define BUG
#ifdef BUG
  mpi_err_handler(MPI_Pack_external(datarep, uint_data, UINT_COUNT,
                                    MPI_UNSIGNED, pack_buffer, pack_size,
                                    &position), MPI_COMM_WORLD);
  mpi_err_handler(MPI_Pack_external(datarep, dble_data, DBLE_COUNT, MPI_DOUBLE,
                                    pack_buffer, pack_size, &position),
                  MPI_COMM_WORLD);
#else // NO_BUG
  mpi_err_handler(MPI_Pack_external(datarep, dble_data, DBLE_COUNT, MPI_DOUBLE,
                                    pack_buffer, pack_size, &position),
                  MPI_COMM_WORLD);
  mpi_err_handler(MPI_Pack_external(datarep, uint_data, UINT_COUNT,
                                    MPI_UNSIGNED, pack_buffer, pack_size,
                                    &position), MPI_COMM_WORLD);
#endif // NO_BUG

  free(pack_buffer);
  free(dble_data);
  free(uint_data);

  MPI_Finalize();

  return EXIT_SUCCESS;
}

//taken from http://beige.ucs.indiana.edu/I590/node85.html
static void mpi_err_handler(int error_code, MPI_Comm comm) {

   if (error_code != MPI_SUCCESS) {

      int rank;

      MPI_Comm_rank(comm, &rank);

      char error_string[1024];
      int length_of_error_string, error_class;

      MPI_Error_class(error_code, &error_class);
      MPI_Error_string(error_class, error_string, &length_of_error_string);
      fprintf(stderr, "%3d: %s\n", rank, error_string);
      MPI_Error_string(error_code, error_string, &length_of_error_string);
      fprintf(stderr, "%3d: %s\n", rank, error_string);
      MPI_Abort(comm, error_code);
   }
}

