#ifndef FORTRAN_DATA_H
#define FORTRAN_DATA_H

#include <stdlib.h>

/** \file read_fortran_data.h
  * \brief general routines for reading Fortran binary file from c
  *
  * These routines should be called in a specific order:
  *
  * -# \ref read_fortran_dimensions
  * -# \ref read_fortran_data
  *
  * \remark These routines are adapted to read the current ICON coupler restart files
  * \remark and are not meant to be general purpose routines. 
 **/

/**
 * reads some metadata from the Fortran file
 * @param[in]  fileId    file pointer to an already open file
 * @param[in]  numCells  expected size of the data
 * @param[out] fieldInfo metadata information about the stored array which has been written into the Fortran file
 */
void read_fortran_dimensions ( FILE *fileId, struct fieldMetadata * fieldInfo, int numCells );

/**
 * reads (Fortran DOUBLE PRECISION) data from the Fortran file
 * @param[out] cell_data guess what?
 * @param[in]  fieldInfo metadata information about the stored array which has been written into the Fortran file
 * @param[out] fileId    file pointer to an already open file
 */
void read_fortran_data ( double * cell_data, struct fieldMetadata fieldInfo, FILE *fileId);

#endif

