package de.mpg.mpimet.yacgui.xml;

import java.util.List;

/**
 * Java representation of {@code <couple>} xml element.
 * 
 * @author Maxim Yastremsky, Teresa Holfeld
 */
public class Couple {
	
	/** ID referencing first component */
	private int component1Id;
	
	/** ID referencing second component */
	private int component2Id;
	
	private List<TransientCouple> transientCouples;

	public Couple(int component1Id, int component2Id, List<TransientCouple> transientCouples) {
		this.component1Id = component1Id;
		this.component2Id = component2Id;
		this.transientCouples = transientCouples;
	}

	public int getComponent1Id() {
		return component1Id;
	}

	public int getComponent2Id() {
		return component2Id;
	}
	
	/**
	 * Returns the component id for component referenced by componentRef property of TransientCouple.
	 * @see TransientCouple#sourceComponentRef
	 * 
	 * @param componentRef
	 * @return component id
	 */
	public int getComponentIdForComponentRef(int componentRef) {
		switch (componentRef) {
		case 1:
			return component1Id;
		case 2:
			return component2Id;
		default:
			throw new IllegalArgumentException("ComponentRef argument is limited only to \"1\" or \"2\" values");	
		}
	}

	public List<TransientCouple> getTransientCouples() {
		return transientCouples;
	}
	
	public void setTransientCouples(List<TransientCouple> transientCouples) {
		this.transientCouples = transientCouples;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + component1Id;
		result = prime * result + component2Id;
		result = prime * result + ((transientCouples == null) ? 0 : transientCouples.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Couple)) {
			return false;
		}
		Couple other = (Couple) obj;
		if (component1Id != other.component1Id) {
			return false;
		}
		if (component2Id != other.component2Id) {
			return false;
		}
		if (transientCouples == null) {
			if (other.transientCouples != null) {
				return false;
			}
		} else if (!transientCouples.equals(other.transientCouples)) {
			return false;
		}
		return true;
	}	
}
