package de.mpg.mpimet.yacgui.xml;

import de.mpg.mpimet.yacgui.utils.Default;

/**
 * Class containing timestep parameters: periods and lags.
 * 
 * @author Maxim Yastremsky
 */
public class Timestep {
	
	//next 3 params will most probably have integers values, but after adding 2 more Timestep's units 
	//(ISO_Format and user string), it became more reasonable to store them as strings
	private String source;	
	private String target;	
	private String couplingPeriod;	
	
	private CouplingOperation couplingOperation;	
	
//	private Timelag sourceTimeLag;
//	private Timelag targetTimeLag;
	private String sourceTimeLag;
	private String targetTimeLag;
	/**
	 * @param source
	 * @param target
	 * @param couplingPeriod
	 * @param couplingOperation
	 * @param sourceTimeLag
	 * @param targetTimeLag
	 */
	public Timestep(String source, String target, String couplingPeriod, String couplingOperation,
			String sourceTimeLag, String targetTimeLag) {
		this.source = source;
		this.target = target;
		this.couplingPeriod = couplingPeriod;
		
		this.couplingOperation = CouplingOperation.objectFromString(couplingOperation);
		
		this.sourceTimeLag = sourceTimeLag;
		this.targetTimeLag = targetTimeLag;
	}
	
	/**
	 * Copy constructor
	 * 
	 * @param donor
	 */
	public Timestep(Timestep donor) {
		this(new String(donor.source), new String(donor.target), new String(donor.couplingPeriod), 
				new String(donor.couplingOperation.toString()), donor.sourceTimeLag, donor.targetTimeLag);
	}
	
	public String getSource() {
		return source;
	}
	
	public String getTarget() {
		return target;
	}
	
	public String getCouplingPeriod() {
		return couplingPeriod;
	}
	
	public CouplingOperation getCouplingOperation() {
		return couplingOperation;
	}
	
	public String getSourceTimeLag() {
		return sourceTimeLag;
	}

	public String getTargetTimeLag() {
		return targetTimeLag;
	}
	
	public void setSource(String source) {
		this.source = source;
	}

	public void setTarget(String target) {
		this.target = target;
	}
	
	public void setCouplingPeriod(String couplingPeriod) {
		this.couplingPeriod = couplingPeriod;
	}
	
	public void setCouplingOperation(String couplingOperation) {
		this.couplingOperation = CouplingOperation.objectFromString(couplingOperation);
	}
	
	public void setSourceTimeLag(String sourceTimeLag) {
		this.sourceTimeLag = sourceTimeLag;
	}
	
	public void setTargetTimeLag(String targetTimeLag) {
		this.targetTimeLag = targetTimeLag;
	}

	

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((couplingOperation == null) ? 0 : couplingOperation
						.hashCode());
		result = prime * result
				+ ((couplingPeriod == null) ? 0 : couplingPeriod.hashCode());
		result = prime * result + ((source == null) ? 0 : source.hashCode());
		result = prime * result + ((sourceTimeLag == null) ? 0 : sourceTimeLag.hashCode());
		result = prime * result + ((target == null) ? 0 : target.hashCode());
		result = prime * result + ((targetTimeLag == null) ? 0 : targetTimeLag.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Timestep other = (Timestep) obj;
		if (couplingOperation != other.couplingOperation)
			return false;
		if (couplingPeriod == null) {
			if (other.couplingPeriod != null)
				return false;
		} else if (!couplingPeriod.equals(other.couplingPeriod))
			return false;
		if (source == null) {
			if (other.source != null)
				return false;
		} else if (!source.equals(other.source))
			return false;
		if (sourceTimeLag != other.sourceTimeLag)
			return false;
		if (target == null) {
			if (other.target != null)
				return false;
		} else if (!target.equals(other.target))
			return false;
		if (targetTimeLag != other.targetTimeLag)
			return false;
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "Timestep [source=" + source + ", target=" + target
				+ ", couplingPeriod=" + couplingPeriod + ", couplingOperation="
				+ couplingOperation + ", sourceTimeLag=" + sourceTimeLag
				+ ", targetTimeLag=" + targetTimeLag + "]";
	}
	
	/**
	 * Returns default Timestep
	 * 
	 * @return default Timestep
	 */
	public static Timestep getDefault() {
		return new Timestep(Default.TIMESTEP_SOURCE, Default.TIMESTEP_TARGET,
				Default.COUPLING_PERIOD, Default.COUPLING_OPERATION.toString(), 
				Default.SOURCE_TIMELAG, Default.TARGET_TIMELAG);
	}
}
