package de.mpg.mpimet.yacgui.xml.interpolation;

/**
 * Enumerates all types of available interpolations
 * 
 * @author Maxim Yastremsky
 */
public enum InterpolationMethod {
	
	AVERAGE("average"),
	N_NEAREST_NEIGHBOR("n-nearest_neighbor"),
	CONSERVATIVE("conservative"),
	RADIAL_BASIS_FUNCTION("radial_basis_function"),
	BERNSTEIN_BEZIER("bernstein_bezier"),
	PATCH_RECOVERY("patch_recovery"),
	FIXED_VALUE("fixed_value"),
	USER_FILE("user_file"),
	SOURCE2TARGET_MAP("source_to_target_map"),
	SMOOTHED_PATCH_RECOVERY("smoothed_patch_recovery");
	
	private final String name;
	
	private InterpolationMethod(String s) {
		name = s;
	}
	
	public String getName() {
		return name;
	}

	public boolean equalsName(String otherName) {
		return (otherName == null) ? false : name.equals(otherName);
	}
	
	/**
	 * check the name is in the list of Interpolation method 
	 * @param name String representation of {@link InterpolationMethod} 
	 * @return true if the name is in the list, false if not
	 */
	public static boolean isValidName(String name) {
		if (name == null) return false;

		if (AVERAGE.equalsName(name)
			|| N_NEAREST_NEIGHBOR.equalsName(name)
	       	|| RADIAL_BASIS_FUNCTION.equalsName(name)
       		|| CONSERVATIVE.equalsName(name)
       		|| BERNSTEIN_BEZIER.equalsName(name)
       		|| PATCH_RECOVERY.equalsName(name)
			|| FIXED_VALUE.equalsName(name)
       		|| USER_FILE.equalsName(name)
			|| SOURCE2TARGET_MAP.equalsName(name)
       		|| SMOOTHED_PATCH_RECOVERY.equalsName(name))
       		return true;
		return false;
	}
	
	/**
	 * select Interpolation method from name when create Interpolation
	 * @param name String representation of {@link InterpolationMethod} 
	 * @return the String representation of selected InterpolationMethod
	 */
	public static InterpolationMethod fromName(String name) {
		if (name == null) return null;
		
		if (AVERAGE.equalsName(name)) return AVERAGE;
		else if (N_NEAREST_NEIGHBOR.equalsName(name)) return N_NEAREST_NEIGHBOR;
		else if (CONSERVATIVE.equalsName(name)) return CONSERVATIVE;
		else if (BERNSTEIN_BEZIER.equalsName(name)) return BERNSTEIN_BEZIER;
		else if (RADIAL_BASIS_FUNCTION.equalsName(name)) return RADIAL_BASIS_FUNCTION;
		else if (PATCH_RECOVERY.equalsName(name)) return PATCH_RECOVERY;
		else if (FIXED_VALUE.equalsName(name)) return FIXED_VALUE;
		else if (USER_FILE.equalsName(name)) return USER_FILE;
		else if (SOURCE2TARGET_MAP.equalsName(name)) return SOURCE2TARGET_MAP;
		else if (SMOOTHED_PATCH_RECOVERY.equalsName(name)) return SMOOTHED_PATCH_RECOVERY;
		return null;
	}
	
	@Override
	public String toString() {
		return getName();
	}
}
