/**
 * @file field_interpolation_gen_src.h
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#ifndef FIELD_INTERPOLATION_GEN_SRC_H
#define FIELD_INTERPOLATION_GEN_SRC_H

#include "field_interpolation_gen.h"

struct field_interpolation_tags;

/**
 * constructor for a field_interpolation_gen on the source side
 * @param[in] points                  point sets
 * @param[in] num_pointsets           number of point sets in points
 * @param[in] field_config_name       name of the local field configuration
 * @param[in] search                  global search used as the basis for the
 *                                    interpolation
 * @param[in] other_field_config_name name of the target field configuration
 *                                    field
 * @param[in] methods                 interpolation method stack that is to be
 *                                    used for the interpolation
 * @param[in] weight_file_name        if != NULL the result of the interpolation
 *                                    will be written to the file
 * @param[in] io_proc_step            stride between ranks of io processes
 *                                    (e.g. for writing weight files)
 * @param[in] mapping_on_source       is the mapping in the put-get-operation to
 *                                    be executed on the source processes (on
 *                                    the target otherwise)
 * @return field interpolation generator
 *
 * @remarks The interpolation starts by applying the first entry in methods to the points
 *          that are to be interpolated. All points that could not be interpolated are
 *          are passed to next interpolation in methods. This is done recursively until
 *          until the last entry in methods is reached.
 * @remarks the last entry in methods has to be NULL
 * @remarks not all interpolations support the writing of the result to file
 *          (e.g. interpolation method fixed)
 */
struct field_interpolation_gen *
yac_new_field_interpolation_gen_src(struct points *** points,
                                    unsigned num_pointsets,
                                    char const * field_config_name,
                                    struct global_search * search,
                                    const char * other_field_config_name,
                                    struct interpolation_method ** methods,
                                    char const * weight_file_name,
                                    unsigned io_proc_step,
                                    int mapping_on_source);

struct field_interpolation_gen *
yac_new_field_interpolation_gen_src_(struct points *** points,
                                     unsigned num_pointsets,
                                     char const * field_config_name,
                                     struct global_search * search,
                                     const char * other_field_config_name,
                                     struct interpolation_method ** methods,
                                     char const * weight_file_name,
                                     unsigned io_proc_step,
                                     struct field_interpolation_tags tags,
                                     int mapping_on_source);

#endif // FIELD_INTERPOLATION_GEN_SRC_H

