/**
 * @file interpolation_method_check.h
 * @brief interpolation method used for debugging the interpolation method
 *        stack
 *
 * @copyright Copyright  (C)  2014 Moritz Hanke <hanke@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#ifndef INTERPOLATION_METHOD_CHECK_H
#define INTERPOLATION_METHOD_CHECK_H

#include "interpolation_method.h"
#include "communicator.h"

typedef void (*func_constructor)(void * user_data);
typedef void (*func_do_search)(int const * global_ids,
  double const (*coordinates_xyz)[3], unsigned num_points, void * user_data);

/**
 * constructor for a interpolation method of type interpolation_method_check
 * @param[in] constructor_callback  pointer to routine that is called in
 *                                  \ref yac_new_interpolation_method_check
 * @param[in] constructor_user_data pointer passed to constructor_callback when
 *                                  it is called
 * @param[in] do_search_callback    pointer to routine that is to be called when
 *                                  the do_search routine of this
 *                                  interpolation_method is called
 * @param[in] do_search_user_data   pointer passed to do_search_callback when it
 *                                  is called
 * @param[in] search                global search object
 * @return returns a pointer to an interpolation method
 */
struct interpolation_method *
yac_new_interpolation_method_check(func_constructor constructor_callback,
                               void * constructor_user_data,
                               func_do_search do_search_callback,
                               void * do_search_user_data,
                               struct global_search * search);

/**
 * sets a constructor_callback that can afterwards be retrieved by
 * \ref yac_interpolation_method_get_constructor_callback
 * @param[in] constructor_callback pointer to a constructor_callback routine
 * @param[in] user_data            pointer to user data
 * @param[in] key                  string that can afterwards be used to
 *                                 retrieve the pointer
 */
void yac_interpolation_method_check_add_constructor_callback(
  func_constructor constructor_callback, void * user_data, char const * key);

/**
 * retrieves a constructor_callback pointer that was set by
 * \ref yac_interpolation_method_check_add_constructor_callback
 * @param[in] key                 string to identify a pointer that was
 *                                previously set
 * @param[out] constructor_callback pointer to constructor_callback routine
 * @param[out] user_data            pointer to user data
 */
void yac_interpolation_method_get_constructor_callback(
  char const * key, func_constructor * constructor_callback, void ** user_data);

/**
 * sets a do_search_callback that can afterwards be retrieved by
 * \ref yac_interpolation_method_get_do_search_callback
 * @param[in] do_search_callback pointer to a do_search_callback routine
 * @param[in] user_data            pointer to user data
 * @param[in] key                string that can afterwards be used to retrieve
 *                               the pointer
 */
void yac_interpolation_method_check_add_do_search_callback(
  func_do_search do_search_callback, void * user_data, char const * key);

/**
 * retrieves a do_search_callback pointer that was set by
 * \ref yac_interpolation_method_check_add_do_search_callback
 * @param[in] key                 string to identify a pointer that was
 *                                previously set
 * @param[out] do_search_callback pointer to do_search_callback routine
 * @param[out] user_data          pointer to user data
 */
void yac_interpolation_method_get_do_search_callback(
  char const * key, func_do_search * do_search_callback, void ** user_data);

#endif // INTERPOLATION_METHOD_CHECK_H

