/**
 * @file interpolation_method_fixed.h
 * @brief interpolation method that assigns a fixed value
 *
 * Interpolates each target point by giving it a predefined fixed value that
 * was provided to \ref yac_new_interpolation_method_fixed.
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#ifndef INTERPOLATION_METHOD_FIXED_H
#define INTERPOLATION_METHOD_FIXED_H

#include "interpolation_method.h"
#include "communicator.h"

/**
 * constructor for a interpolation method of type interpolation_method_fixed
 * @param[in] fixed_value value that is to be assigned to all points that need
 *                        to be interpolated
 * @param[in] search global search object
 * @return returns a pointer to an interpolation method
 */
struct interpolation_method *
yac_new_interpolation_method_fixed(double fixed_value,
                                   struct global_search * search);

#endif // INTERPOLATION_METHOD_FIXED_H

