/**
 * @file mask.c
 * @brief Mask data structure
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 */

#ifndef MASK_H
#define MASK_H

#include "points.h"

/** \example test_mask.c
 * This shows how to work with struct mask.
 */

struct mask {

   unsigned * mask; //!< array that contains entry for each data point on
                    //!< the grid (!= 0 => points is masked)
   unsigned unique_id; //!< locally unique id
};

/**
  * initialises mask data struct
  * @param[in,out] mask        mask data struct that is to be initialised
  * @param[in]     mask_values contains for each point the mask value
  * @param[in]     points      points set the mask is associated with
  */
void yac_init_mask(struct mask * mask, unsigned const * mask_values,
                   struct points * points);

/**
 * frees all memory associated with the given mask object
 * @param[in] mask
 */
void yac_free_mask(struct mask * mask);

#endif // MASK_H

