//#define VERBOSE
/**
 * @file test_events.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 *
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * YAC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include "tests.h"
#include "event.h"
#include "utils.h"

int main (void) {

  int event_id[2];

  void * model_time_step_Ptr;
  void * coupling_time_step_Ptr;

  int model_time_step;
  int coupling_time_step;

  char * model_time_step_iso;
  char * coupling_time_step_iso;

  int lag;

  enum yac_reduction_type time_operation;
  enum yac_time_unit_type time_unit;

  int i;
  int action;
  int action0[12] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 2};
  int action1[12] = { 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 2};

  calendarType set_calendar = getCalendarType ();
  calendarType my_calendar  = PROLEPTIC_GREGORIAN;

  initCalendar(my_calendar);

  if ( set_calendar == CALENDAR_NOT_SET )
     initCalendar(my_calendar);
  else if ( set_calendar != my_calendar ) {
     yac_internal_abort_message("ERROR: different calendars in use! Reconfigure model setup.\n",  __FILE__, __LINE__);
     PUT_ERR("wrong action\n");
  }

  /* --------------- First event ------------------ */

  yac_event_init ( &event_id[0] );

  model_time_step    = 5000;
  coupling_time_step = 15000;

  model_time_step_Ptr = &model_time_step;
  coupling_time_step_Ptr = &coupling_time_step;

  lag                = 0;
  time_operation     = TIME_AVERAGE;
  time_unit          = C_MILLISECOND;

  yac_event_add ( event_id[0], model_time_step_Ptr, coupling_time_step_Ptr,
		  time_unit, lag, time_operation,
                  "1850-01-01T00:00:00",
                  "1850-01-01T00:01:00");

  /* --------------- Second event ----------------- */

  yac_event_init ( &event_id[1] );

  model_time_step_iso = "PT30M";
  coupling_time_step_iso = "PT60M";

  model_time_step_Ptr = model_time_step_iso;
  coupling_time_step_Ptr = coupling_time_step_iso;

  lag                = 0;
  time_operation     = TIME_AVERAGE;
  time_unit          = C_ISO_FORMAT;

  yac_event_add ( event_id[1], model_time_step_Ptr, coupling_time_step_Ptr,
                 time_unit, lag, time_operation,
                 "1850-01-01T00:00:00",
                 "1850-01-01T06:00:00");

  /* --------------- Check what's inside ---------- */

#ifdef VERBOSE
  printf ( " ------------------------------------------------------ \n" );
#endif

  /* Event 0 : 60 seconds run time, 5 seconds per time step -> 12 time steps*/

  for ( i = 0; i < 12; i++ ) {
     yac_event_update ( event_id[0] );
     action = yac_event_check ( event_id[0] );
#ifdef VERBOSE
     if ( action == COUPLING )
       printf ("event check: time for coupling for event %d \n", event_id[0]);
     if ( action == RESTART )
       printf ("event check: time for restart for event  %d \n", event_id[0]);
#endif
     if ( action != action0[i] ) PUT_ERR("wrong action\n");
  }

  /* Event 1 : 6 hours run time, 1800 seconds per time step -> 12 time steps*/

#ifdef VERBOSE
  printf ( " ------------------------------------------------------ \n" );
#endif

  for ( i = 0; i < 12; i++ ) {
     yac_event_update ( event_id[1] );
     action = yac_event_check ( event_id[1] );
#ifdef VERBOSE
     if ( action == COUPLING )
       printf ("event check: time for coupling for event %d \n", event_id[1]);
     if ( action == RESTART )
       printf ("event check: time for restart for event  %d \n", event_id[1]);
#endif
     if ( action != action1[i] ) PUT_ERR("wrong action\n");
  }

  for ( i = 0; i < 2; i++ )
     yac_event_free ( event_id[i] );

  freeCalendar();
  yac_free_pointer_unique_lookup();

  return TEST_EXIT_CODE;
}


